#!/usr/bin/awk -f
#* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
#*                                                                           *
#*            This file is part of the test engine for MIPLIB2010            *
#*                                                                           *
#* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
# $Id: parse.awk,v 1.13 2011/05/05 17:37:10 bzfgamra Exp $

function abs(x)
{
   return x < 0 ? -x : x;
}
function min(x,y)
{
   return (x) < (y) ? (x) : (y);
}
function max(x,y)
{
   return (x) > (y) ? (x) : (y);
}
BEGIN {
   printf("----------------------------+--------+----------------+----------------+------+----------+-------+---------\n");
   printf("Name                        |  Type  |   Dual Bound   |  Primal Bound  | Gap%% | Integral |  Time |  Status \n");
   printf("----------------------------+--------+----------------+----------------+------+----------+-------+---------\n");

   infty = +1e+20;
   eps = 1e-04;
   largegap = 1e+04;
   reltol = 1e-06

   # initialize summary data
   nsolved = 0;
   nstopped = 0;
   nfailed = 0;

   solver = "?";
   score = "?";
}

# parse solution status from solution file (optional)
/=opt=/  { knownsolstatus[$2] = "opt"; knownsolvalue[$2] = $3; }   # get optimum
/=inf=/  { knownsolstatus[$2] = "inf"; }                      # problem infeasible (no feasible solution exists)
/=best=/ { knownsolstatus[$2] = "best"; knownsolvalue[$2] = $3; }  # get best known solution value
/=feas=/ { knownsolstatus[$2] = "feas"; }                     # no feasible solution known
/=unkn=/ { knownsolstatus[$2] = "unkn"; }                     # no feasible solution known

# parse benchmark code score
/^SCORE:/{ score = $2; }


# instance name
/^@01/ {
   n  = split ($2, a, "/");
   m = split(a[n], b, ".");
   prob = b[1];
   if( b[m] == "gz" || b[m] == "z" || b[m] == "GZ" || b[m] == "Z" )
      m--;
   for( i = 2; i < m; ++i )
      prob = prob "." b[i];

   # initialize data to be set in parse.awk
   timelimit = 0;
   starttime = 0.0;
   endtime = 0.0;
   time = 0.0;
   pb = +infty;
   db = -infty;
   aborted = 1;
   timeout = 0;

   type = "?";
   lastprimal = infty;
   lasttime = 0.0;
   primalint = 0.0;
}

# solver
/^Program/ {solver = $2;}
/^Problem/ {type = $2;}


# time, dual, primal
/^@02/ { timelimit = 1.0*$2; }
/^Time/ { time = 1.0*$2; aborted = 0;}
/^Dual/ { db = 1.0*$2; }
/^Primal/ { pb = 1.0*$2; }

# solutions
/^Solution/ {
   soltime = 1.0*$2;
   solprimal = 1.0*$3;

   if( lastprimal > infty/2 )
   {
      primalint += (soltime - lasttime);
   }
   else if( abs(solprimal - knownsolvalue[prob]) > eps )
   {
      primalgap = (solprimal - knownsolvalue[prob]) / max(solprimal, knownsolvalue[prob]);
      primalint += primalgap * (soltime - lasttime);
   }

#   printf("soltime: %7.3f primal: %7.3f lasttime: %7.3f lastprimal %7.3f primalint: %7.3f\n",
#	  soltime, solprimal, lasttime, lastprimal, primalint);

   lasttime = soltime;
   lastprimal = solprimal;
}
/^=ready=/ {

   # add last section of primal integral
   if( lastprimal > infty/2 )
   {
      primalint += (timelimit - lasttime);
   }
   else if( abs(solprimal - knownsolvalue[prob]) > eps )
   {
      primalgap = (solprimal - knownsolvalue[prob]) / max(solprimal, knownsolvalue[prob]);
      primalint += primalgap * (timelimit - lasttime);
   }


  # compute gap
   temp = pb;
   pb = 1.0*temp;
   temp = db;
   db = 1.0*temp;

   if( abs(pb - db) < eps && pb < +infty )
      gap = 0.0;
   else if( abs(db) < eps || abs(pb) < eps)
      gap = -1.0;
   else if( pb*db < 0.0 )
      gap = -1.0;
   else if( abs(db) >= +infty )
      gap = -1.0;
   else if( abs(pb) >= +infty )
      gap = -1.0;
   else
      gap = 100.0*abs((pb-db)/min(abs(db), abs(pb)));

   if( gap < 0.0 )
      gapstr = "    --";
   else if( gap < largegap )
      gapstr = sprintf("%6.1f", gap);
   else
      gapstr = " Large";

   if( abs(pb - db) > eps )
      timeout = 1;

   # determine solving status
   status = "";
   if( aborted )
      status = "abort";
   else if( timeout )
      status = "stopped";
   else
      status = "ok";

   # instance solved correctly (including case that no solution was found)
   if( status == "ok" )
      nsolved++;
   # incorrect solving process
   else if( status == "abort" )
      nfailed++;
   # stopped due to imposed limits
   else
      nstopped++;


   if( prob in knownsolstatus )
   {
      intstr = sprintf("%10.2f", primalint);
   }
   else
   {
      intstr = "?";
   }

   printf("%-28s %8s %16.9g %16.9g %6s %10s %7.1f %8s\n", prob, type, db, pb, gapstr, intstr, time, status);
}
END {
   printf("----------------------------+----------------+----------------+------+---------+-------+--------+---------\n");
   printf("\n");
   printf("solved/stopped/failed: %d/%d/%d\n", nsolved, nstopped, nfailed);
   printf("\n");
   printf("@01 score %s\n", score);
   printf("@02 solver %s\n", solver);
   printf("@03 timelimit: %g\n", timelimit);
}
